//Edmonds Karp

#define NN 1024

int cap[NN][NN];
int cost[NN][NN];
int fnet[NN][NN], adj[NN][NN], deg[NN];
int par[NN], d[NN];
int pi[NN];

#define CLR(a, x) memset( a, x, sizeof( a ) )
#define Inf (INT_MAX/2)
#define Pot(u,v) (d[u] + pi[u] - pi[v])
bool dijkstra( int n, int s, int t )
{
	for( int i = 0; i < n; i++ ) d[i] = Inf, par[i] = -1;
	d[s] = 0;
	par[s] = -n - 1;

	while( 1 )
	{
		int u = -1, bestD = Inf;
		for( int i = 0; i < n; i++ ) if( par[i] < 0 && d[i] < bestD )
			bestD = d[u = i];
		if( bestD == Inf ) break;

		par[u] = -par[u] - 1;
		for( int i = 0; i < deg[u]; i++ )
		{
			int v = adj[u][i];
			if( par[v] >= 0 ) continue;
			if( fnet[v][u] && d[v] > Pot(u,v) - cost[v][u] )
				d[v] = Pot( u, v ) - cost[v][u], par[v] = -u-1;

			if( fnet[u][v] < cap[u][v] && d[v] > Pot(u,v) + cost[u][v] )
				d[v] = Pot(u,v) + cost[u][v], par[v] = -u - 1;

		}

	}

	for( int i = 0; i < n; i++ ) if( pi[i] < Inf ) pi[i] += d[i];

	return par[t] >= 0;
}
#undef Pot
int mcmf3( int n, int s, int t, int &fcost )
{
	CLR( deg, 0 );
	for( int i = 0; i < n; i++ )
		for( int j = 0; j < n; j++ )
			if( cap[i][j] || cap[j][i] ) adj[i][deg[i]++] = j;

	CLR( fnet, 0 );
	CLR( pi, 0 );
	int flow = fcost = 0;

	while( dijkstra( n, s, t ) )
	{
		int bot = INT_MAX;
		for( int v = t, u = par[v]; v != s; u = par[v = u] )
			bot <?= fnet[v][u] ? fnet[v][u] : ( cap[u][v] - fnet[u][v] );

		for( int v = t, u = par[v]; v != s; u = par[v = u] )
			if( fnet[v][u] ) { fnet[v][u] -= bot; fcost -= bot * cost[v][u]; }
			else { fnet[u][v] += bot; fcost += bot * cost[u][v]; }

		flow += bot;
	}

	return flow;
}
int main()
{
	int numV;
	cin >> numV;
	memset( cap, 0, sizeof( cap ) );

	int m, a, b, c, cp;
	int s, t;
	cin >> m;
	cin >> s >> t;

	for (int i=0; i<m; i++) {
		cin >> a >> b >> cp >> c;
		cost[a][b] = c;
		cap[a][b] = cp;
	}

	int fcost;
	int flow = mcmf3( numV, s, t, fcost );
	cout << "flow: " << flow << endl;
	cout << "cost: " << fcost << endl;

	return 0;
}
