// Intersection
const double eps=1e-7;
template<class T> inline T sqr(T x){return x*x;}
double dist(double x1,double y1,double x2,double y2){return sqrt(sqr(x1-x2)+sqr(y1-y2));}
template<class T> T cross(T x0,T y0,T x1,T y1,T x2,T y2){return (x1-x0)*(y2-y0)-(x2-x0)*(y1-y0);}
int crossOper(double x0,double y0,double x1,double y1,double x2,double y2)
{double t=(x1-x0)*(y2-y0)-(x2-x0)*(y1-y0);if (fabs(t)<=eps) return 0;return (t<0)?-1:1;}
bool isIntersect(double x1,double y1,double x2,double y2,double x3,double y3,double x4,double y4)
{return crossOper(x1,y1,x2,y2,x3,y3)*crossOper(x1,y1,x2,y2,x4,y4)<0 &&
	crossOper(x3,y3,x4,y4,x1,y1)*crossOper(x3,y3,x4,y4,x2,y2)<0;}
	const int maxsize=200+5;
	struct Point
{
	double x,y;
};
struct Line
{
	double a,b,c;
	Line(double=0,double=0,double=0);
};
Line::Line(double a0,double b0,double c0)
{
	a=a0;b=b0;c=c0;
}
struct Circle
{
	double x,y,r;
	Circle(double=0,double=0,double=0);
};
Circle::Circle(double x0,double y0,double r0)
{
	x=x0;y=y0;r=r0;
}
int n;
Point P[maxsize];
int nL,nC;
Line L[maxsize];
Circle C[maxsize];
double R1,R2;
Line getLine(Point p1,Point p2)
{
	double a=+(p1.y-p2.y);
	double b=-(p1.x-p2.x);
	double c=p1.x*p2.y-p2.x*p1.y;
	double d=sqrt(sqr(a)+sqr(b));
	a/=d;
	b/=d;
	c/=d;
	return Line(a,b,c);
}
double dist(Point a,Point b)
{
	return dist(a.x,a.y,b.x,b.y);
}
double cross(Point a,Point b,Point c)
{
	return cross(a.x,a.y,b.x,b.y,c.x,c.y);
}
bool isIntersect(Point a,Point b,Point c,Point d)
{
	return isIntersect(a.x,a.y,b.x,b.y,c.x,c.y,d.x,d.y);
}
int getIntersect(Line A,Line B,Point &P)
{
	double a1=A.a;
	double b1=A.b;
	double c1=A.c;
	double a2=B.a;
	double b2=B.b;
	double c2=B.c;
	double t=a1*b2-a2*b1;
	if (fabs(t)<=eps) return 0;
	double X0=+(b1*c2-b2*c1)/t;
	double Y0=-(a1*c2-a2*c1)/t;
	P.x=X0;
	P.y=Y0;
	return 1;
}
int getIntersect(Line A,Circle C,Point &P,Point &Q)
{
	double a=A.a;
	double b=A.b;
	double c=A.c;
	double CX=C.x,CY=C.y;
	double R=C.r;
	//ax+by+c=0
	//(+by+c+aCX)^2+(ay-aCY)^2=(aR)^2
	double x1,y1,x2,y2;
	if (fabs(a)>fabs(b))
	{
		double A=sqr(a)+sqr(b);
		double B=2.0*b*(c+a*CX)-2.0*sqr(a)*CY;
		double C=sqr(c+a*CX)+sqr(a)*(sqr(CY)-sqr(R));
		double delta=sqr(B)-4*A*C;
		if (delta<-eps) return 0;
		if (delta<0) delta=0;
		delta=sqrt(delta);
		y1=(-B+delta)/(2*A);x1=(-c-b*y1)/a;
		y2=(-B-delta)/(2*A);x2=(-c-b*y2)/a;
		P.x=x1;P.y=y1;
		Q.x=x2;Q.y=y2;
	}
	else
	{
		swap(a,b);swap(CX,CY);
		double A=sqr(a)+sqr(b);
		double B=2.0*b*(c+a*CX)-2.0*sqr(a)*CY;
		double C=sqr(c+a*CX)+sqr(a)*(sqr(CY)-sqr(R));
		double delta=sqr(B)-4*A*C;
		if (delta<-eps) return 0;
		if (delta<0) delta=0;
		delta=sqrt(delta);
		y1=(-B+delta)/(2*A);x1=(-c-b*y1)/a;
		y2=(-B-delta)/(2*A);x2=(-c-b*y2)/a;
		swap(x1,y1);swap(x2,y2);
		swap(a,b);swap(CX,CY);
		P.x=x1;P.y=y1;
		Q.x=x2;Q.y=y2;
	}
	return 2;
}
int getIntersect(Circle A,Circle B,Point &P,Point &Q)
{
	double X1=A.x,Y1=A.y,X2=B.x,Y2=B.y;
	double R1=A.r,R2=B.r;
	double dst=dist(X1,Y1,X2,Y2);
	if (dst>R1+R2+eps || dst<fabs(R1-R2)-eps) return 0;
	double a=X1-X2;
	double b=Y1-Y2;
	double c=-(a*(X1+X2)+b*(Y1+Y2)-sqr(R1)+sqr(R2))/2.0;
	double CX=X1,CY=Y1;
	double x1,y1,x2,y2;
	if (fabs(a)>fabs(b))
	{
		double A=sqr(a)+sqr(b);
		double B=2.0*b*(c+a*CX)-2.0*sqr(a)*CY;
		double C=sqr(c+a*CX)+sqr(a)*(sqr(CY)-sqr(R1));
		double delta=sqr(B)-4*A*C;
		if (delta<-eps) return 0;
		if (delta<0) delta=0;
		delta=sqrt(delta);
		y1=(-B+delta)/(2*A);x1=(-c-b*y1)/a;
		y2=(-B-delta)/(2*A);x2=(-c-b*y2)/a;
		P.x=x1;P.y=y1;
		Q.x=x2;Q.y=y2;
	}
	else
	{
		swap(a,b);swap(CX,CY);
		double A=sqr(a)+sqr(b);
		double B=2.0*b*(c+a*CX)-2.0*sqr(a)*CY;
		double C=sqr(c+a*CX)+sqr(a)*(sqr(CY)-sqr(R1));
		double delta=sqr(B)-4*A*C;
		if (delta<-eps) return 0;
		if (delta<0) delta=0;
		delta=sqrt(delta);
		y1=(-B+delta)/(2*A);x1=(-c-b*y1)/a;
		y2=(-B-delta)/(2*A);x2=(-c-b*y2)/a;
		swap(x1,y1);swap(x2,y2);
		swap(a,b);swap(CX,CY);
		P.x=x1;P.y=y1;
		Q.x=x2;Q.y=y2;
	}
	return 2;
}

