/* ================================================================
`\itshape 二维几何`
*/
const double pi = acos(0.0) * 2.0;

struct Tpoint
{
	double x , y;
	Tpoint(){}
	Tpoint(double x0 , double y0){ x = x0; y = y0; }

	// `\itshape 逆时针旋转度`
	void	rotate90()
	{
		double	t = x;
		x = - y; y = t;
	}

	// `\itshape 非朴素法逆时针旋转theta角`
	void	rotate(double theta)	
	{
		double x0 = x;
		double y0 = y;

		theta = - theta;
		x = x0 * cos(theta) + y0 * sin(theta);
		y = - x0 * sin(theta) + y0 * cos(theta);
	}
};

//`\itshape 求直线AB , PQ的交点`
Tpoint	crossPoint(const Tpoint& A , const Tpoint& B , const Tpoint& P , const Tpoint& Q)
{
	double t = ((A.x - P.x) * (B.y - A.y) - (A.y - P.y) * (B.x - A.x))
		/ ((Q.x - P.x) * (B.y - A.y) - (Q.y - P.y) * (B.x - A.x));
	return	Tpoint( P.x + (Q.x - P.x) * t , P.y + (Q.y - P.y) * t );
}

/*
`\itshape 点P到直线AB距离`
	V = B - A; V.rotate90();
	`\itshape 垂足H = crossPoint(P , P + V , A , B);`
	`\itshape 距离d = dist(H , P);`

`\itshape 直线与圆的交点  	:	应先判距离!!!!!!!!!!!!!!!!!`
`\itshape 圆与圆的交点		:	应先判内含, 相离!!!!!!!!!!!!`
*/

`\itshape 直线方程  参数式 与 一般式 的转换`
`\begin{eqnarray*}
\begin{array}{c}
	x=P_x+td_x\\
	y=P_y+td_y
\end{array} &\to& -d_yx+d_xy+(P_xd_y-P_yd_x)=0\\
ax+by+c=0 &\to& \begin{array}{c}
P=\left[\frac{\displaystyle-ac}{\displaystyle a^2+b^2},\frac{\displaystyle-bc}{\displaystyle a^2+b^2}\right]\\
\vec{c}=[-b,a]
\end{array}
\end{eqnarray*}`


/* ================================================================
`\itshape 半平面交` Half-plane O(N2)
*/
#include <cstdio>
#include <cstring>
#include <cmath>

struct			Thalf_plane
{
	Tpoint		p1 , p2; // `\itshape p1->p2的右边为有效区域`
};

int					n;
Thalf_plane		list	[limitSize];
double				area;

int				m;
Tpoint			polygon	[limitSize];	// `\itshape 最终结果 [0,m-1]`
Tpoint			tmp	[limitSize];
bool			valid	[limitSize];

void	half_plane_solve()
{
	m = 0;
	polygon[m ++] = Tpoint(0,0);
	polygon[m ++] = Tpoint(10000,0);
	polygon[m ++] = Tpoint(10000,10000);
	polygon[m ++] = Tpoint(0,10000);

	int	i , j , c;
	for (i = 0; i < n; i ++)
	{
		polygon[m] = polygon[0]; c = 0;
		for (j = 0; j <= m; j ++)
		{
			valid[j] = crossProduct(list[i].p1, list[i].p2, polygon[j])<=0;
			if (j && (valid[j] != valid[j - 1]))
				tmp[c ++] = crossPoint(list[i].p1, list[i].p2, polygon[j-1], polygon[j]);
			if (j < m && valid[j])
				tmp[c ++] = polygon[j];
		}
		m = c;
		for (j = 0; j < m; j ++)
			polygon[j] = tmp[j];
		if (m < 3) break;
	}

	polygon[m] = polygon[0];

	area = 0;
	for (i = 0; i < m; i ++)
		area += polygon[i].x * polygon[i + 1].y - polygon[i].y * polygon[i + 1].x;
	area /= 2.0;
}




/* ================================================================
`\itshape 半平面交 Half-plane O(NlgN) (可能有精度问题)`
*/
#include <cstdio>
#include <cstring>
#include <cmath>
#include <algorithm>
using namespace std;

const	int		limitSize	= 20000 + 10;
const	double		eps		= 1e-8;

struct			Tpoint
{
	double		x , y;
	Tpoint(){}
	Tpoint(double x0 , double y0) : x(x0), y(y0) {}
};

inline	int	crossProduct(const Tpoint& p0 , const Tpoint& p1 , const Tpoint& p2)
{
	return	sgn( (p1.x - p0.x) * (p2.y - p0.y) - (p1.y - p0.y) * (p2.x - p0.x) );
}

bool	segmentCross(const Tpoint& A , const Tpoint& B, const Tpoint& P, const Tpoint& Q)
{
	return	crossProduct(A,B,P) * crossProduct(A,B,Q) <= 0
		&& crossProduct(P,Q,A) * crossProduct(P,Q,B) <= 0;
}

struct			Thalf_plane
{
	Tpoint		p1 , p2; // `\itshape p1->p2的右边为有效区域`
	Tpoint		v;	 // `\itshape v = p1->p2`
	double		angle;
};

int			n;
Thalf_plane		list	[limitSize];
double			area;

void	init()
{
	scanf("%d" , &n);
	for (int i = 0; i < n; i ++)
	{
		scanf("%lf%lf%lf%lf" , &list[i].p2.x , &list[i].p2.y, 
			&list[i].p1.x , &list[i].p1.y);
		list[i].v = list[i].p2 - list[i].p1;
	}
	list[n].p1 = Tpoint(0,0); list[n].p2 = Tpoint(0,10000);
	list[n].v = list[n].p2 - list[n].p1; n ++;
	list[n].p1 = Tpoint(0,10000); list[n].p2 = Tpoint(10000,10000);
	list[n].v = list[n].p2 - list[n].p1; n ++;
	list[n].p1 = Tpoint(10000,10000); list[n].p2 = Tpoint(10000,0);
	list[n].v = list[n].p2 - list[n].p1; n ++;
	list[n].p1 = Tpoint(10000,0); list[n].p2 = Tpoint(0,0);
	list[n].v = list[n].p2 - list[n].p1; n ++;
}

bool	operator < (const Thalf_plane& A , const Thalf_plane& B)
{
	if (cmp(A.angle , B.angle)) return cmp(A.angle , B.angle) > 0;	
	return	crossProduct(A.p1 , A.p2 , B.p1) > 0;
}

int			S , T;
int			stack	[limitSize];
Tpoint			point	[limitSize];

void	half_plane_solve()
{
	int		i;

	for (i = 0; i < n; i ++)
		list[i].angle = atan2(list[i].v.y , list[i].v.x);
	sort(list , list + n);

	double	leng = 1e20 / sqrt(list[0].v.x * list[0].v.x + list[0].v.y * list[0].v.y); //`\itshape 设一个比较长的距离`
	point[0].x = list[0].p1.x - list[0].v.x * leng;
	point[0].y = list[0].p1.y - list[0].v.y * leng;
	stack[S = T = 0] = 0;

	for (i = 1; i < n; i ++)
	{
		if ( sgn(list[i].v.x * list[i - 1].v.y - list[i].v.y * list[i - 1].v.x) )	// `\itshape 不平行`
		{
			while (crossProduct(list[i].p1 , point[T] , list[i].p2) <= 0 )
			{
				T --;
				if (T < S){ area = 0; return; }
			}
			point[T + 1] = crossPoint(list[stack[T]].p1, list[stack[T]].p2, list[i].p1, list[i].p2);
			stack[++ T] = i;
			while ( S < T && crossProduct(list[i].p1 , point[S] , list[i].p2) <= 0 ) 
				S ++;
		}
	}
	S --;
	if (S < 0) while(1){}

	for (i = S + 2; i < T; i ++)
		if (segmentCross(point[S], point[S+1], point[i], point[i+1]))
			T = i;
	//`\itshape 最终凸包为point[S..T-1] , 且point[S] = point[T]`
	point[S] = crossPoint(list[stack[T]].p1, list[stack[T]].p2, point[S], point[S+1]);
	point[++ T] = point[S];
	area = 0;
	for (i = S; i < T; i ++)
		area += point[i].x * point[i + 1].y - point[i].y * point[i + 1].x;
	area = - area / 2.0;
}

